// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

#include "RuntimeMeshLoader.h"

#define LOCTEXT_NAMESPACE "FRuntimeMeshLoaderModule"

void FRuntimeMeshLoaderModule::StartupModule()
{
	// This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module
	// Get the base directory of this plugin
	BaseDir = FPaths::ProjectPluginsDir() + "RuntimeMeshLoader";

	// Add on the relative location of the third party dll and load it
	//FString LibraryPath, Dual_Camera_Device_Intrinsic_LibPath, opencvLibPath;
#if PLATFORM_WINDOWS
	const FString LibName = "assimp-vc141-mt.dll";
#elif PLATFORM_MAC
	//LibraryPath = FPaths::Combine(*BaseDir, TEXT("Source/ThirdParty/SRWorksLibrary/Mac/Release/libExampleLibrary.dylib"));
#endif // PLATFORM_WINDOWS

	LoadLibraryDLL(LibName, DllHandle);

	if (DllHandle == nullptr)
	{
		FMessageDialog::Open(EAppMsgType::Ok, LOCTEXT("ThirdPartyLibraryError", "Failed to load assimp library"));
	}
}

bool FRuntimeMeshLoaderModule::LoadLibraryDLL(const FString& libName, void*& handle)
{
	FString LibraryPath = FPaths::Combine(*BaseDir, TEXT("Binaries/Win64"), *libName);
	handle = !LibraryPath.IsEmpty() ? FPlatformProcess::GetDllHandle(*LibraryPath) : nullptr;
	return handle != nullptr;
}

void FRuntimeMeshLoaderModule::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FRuntimeMeshLoaderModule, RuntimeMeshLoader)