// Fill out your copyright notice in the Description page of Project Settings.

#include "ViveSR_DualCameraImageRenderer.h"
#include "ViveSR.h"
#include "ViveSR_DualCameraImageCapture.h"
#include "ViveSR_DualCameraImagePlane.h"
#include "ConstructorHelpers.h"
#include "Engine.h"


AViveSR_DualCameraImageRenderer::AViveSR_DualCameraImageRenderer()
{
	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;
	PrimaryActorTick.bStartWithTickEnabled = false;

	root_arrow = CreateDefaultSubobject<UArrowComponent>(TEXT("Arrow"));
	RootComponent = root_arrow;

	static ConstructorHelpers::FObjectFinder<UMaterialInstanceConstant> distortedMatInstAssetLeft(TEXT("MaterialInstanceConstant'/SRWorks/Materials/DistortedMaterialLeft_Inst.DistortedMaterialLeft_Inst'"));
	if (distortedMatInstAssetLeft.Succeeded())
	{
		DistortedMaterialInstanceLeft = distortedMatInstAssetLeft.Object;
	}
	static ConstructorHelpers::FObjectFinder<UMaterialInstanceConstant> distortedMatInstAssetRight(TEXT("MaterialInstanceConstant'/SRWorks/Materials/DistortedMaterialRight_Inst.DistortedMaterialRight_Inst'"));
	if (distortedMatInstAssetRight.Succeeded())
	{
		DistortedMaterialInstanceRight = distortedMatInstAssetRight.Object;
	}
	static ConstructorHelpers::FObjectFinder<UMaterialInstanceConstant> undistortedMatInstAssetLeft(TEXT("MaterialInstanceConstant'/SRWorks/Materials/UndistortedMaterialLeft_Inst.UndistortedMaterialLeft_Inst'"));
	if (undistortedMatInstAssetLeft.Succeeded())
	{
		UndistortedMaterialInstanceLeft = undistortedMatInstAssetLeft.Object;
	}
	static ConstructorHelpers::FObjectFinder<UMaterialInstanceConstant> undistortedMatInstAssetRight(TEXT("MaterialInstanceConstant'/SRWorks/Materials/UndistortedMaterialRight_Inst.UndistortedMaterialRight_Inst'"));
	if (undistortedMatInstAssetRight.Succeeded())
	{
		UndistortedMaterialInstanceRight = undistortedMatInstAssetRight.Object;
	}
	static ConstructorHelpers::FObjectFinder<UMaterialInstanceConstant> DePurpleMatInstAssetLeft(TEXT("MaterialInstanceConstant'/SRWorks/Materials/dePurple_Mat_Inst_Left.dePurple_Mat_Inst_Left'"));
	if (DePurpleMatInstAssetLeft.Succeeded())
	{
		DePurpleMaterialInstanceLeft = DePurpleMatInstAssetLeft.Object;
	}
	static ConstructorHelpers::FObjectFinder<UMaterialInstanceConstant> DePurpleMatInstAssetRight(TEXT("MaterialInstanceConstant'/SRWorks/Materials/dePurple_Mat_Inst_Right.dePurple_Mat_Inst_Right'"));
	if (DePurpleMatInstAssetRight.Succeeded())
	{
		DePurpleMaterialInstanceRight = DePurpleMatInstAssetRight.Object;
	}

	static ConstructorHelpers::FObjectFinder<UMaterialInstanceConstant> depthMatInstAsset(TEXT("MaterialInstanceConstant'/SRWorks/Materials/DepthMaterial_Inst.DepthMaterial_Inst'"));
	if (depthMatInstAsset.Succeeded())
	{
		DepthMaterialInstance = depthMatInstAsset.Object;
	}
	else
	{
		UE_LOG(LogViveSR, Warning, TEXT("Default Depth Material Instance not found."));
	}
}

void AViveSR_DualCameraImageRenderer::SetMode(DualCameraDisplayMode mode)
{
	DisplayMode = mode;
	static_cast<AViveSR_DualCameraImagePlane*>(image_plane)->SetMode(mode);

	switch (mode)
	{
	case DualCameraDisplayMode::MIX:
		SetActorTickEnabled(true);
		break;
	case DualCameraDisplayMode::VIRTUAL:
	default:
		SetActorTickEnabled(false);
		break;
	}
}
void AViveSR_DualCameraImageRenderer::Initial()
{
    // Initial ViveSR_DualCameraImageCapture
    ViveSR_DualCameraImageCapture::Instance()->SetGPUMethod(GPUMethod);
    ViveSR_DualCameraImageCapture::Instance()->Initial();

    distorted_img_width = ViveSR_DualCameraImageCapture::Instance()->distorted_img_width;
    distorted_img_height = ViveSR_DualCameraImageCapture::Instance()->distorted_img_height;
    undistorted_img_width = ViveSR_DualCameraImageCapture::Instance()->undistorted_img_width;
    undistorted_img_height = ViveSR_DualCameraImageCapture::Instance()->undistorted_img_height;
    camera_status = DualCameraStatus::WORKING;

    DistortedDynamicMaterialSelfLeft = UMaterialInstanceDynamic::Create(DistortedMaterialInstanceLeft, this);
    DistortedDynamicMaterialSelfRight = UMaterialInstanceDynamic::Create(DistortedMaterialInstanceRight, this);
    UndistortedDynamicMaterialSelfLeft = UMaterialInstanceDynamic::Create(UndistortedMaterialInstanceLeft, this);
    UndistortedDynamicMaterialSelfRight = UMaterialInstanceDynamic::Create(UndistortedMaterialInstanceRight, this);
    DePurpleDynamicMaterialSelfLeft = UMaterialInstanceDynamic::Create(DePurpleMaterialInstanceLeft, this);
    DePurpleDynamicMaterialSelfRight = UMaterialInstanceDynamic::Create(DePurpleMaterialInstanceRight, this);
    DepthDynamicMaterialSelf = UMaterialInstanceDynamic::Create(DepthMaterialInstance, this);
    is_horus = ViveSR_DualCameraImageCapture::Instance()->IsHorus();
    need_depurple = is_horus && PurpleFringeRemoval;
   
    UndistortedDynamicMaterialsLeft.Add(need_depurple ? DePurpleDynamicMaterialSelfLeft : UndistortedDynamicMaterialSelfLeft);
    UndistortedDynamicMaterialsRight.Add(need_depurple? DePurpleDynamicMaterialSelfRight: UndistortedDynamicMaterialSelfRight);

    DistortedDynamicMaterialsLeft.Add(DistortedDynamicMaterialSelfLeft);
    DistortedDynamicMaterialsRight.Add(DistortedDynamicMaterialSelfRight);  
    DepthDynamicMaterials.Add(DepthDynamicMaterialSelf);

    AViveSR_DualCameraImagePlane* planeActor = GetWorld()->SpawnActor<AViveSR_DualCameraImagePlane>();
    planeActor->AttachToActor(this, FAttachmentTransformRules::KeepRelativeTransform);
    planeActor->Initial();
    planeActor->DistortedImagePlaneLeft->SetMaterial(0, DistortedDynamicMaterialSelfLeft);
    planeActor->DistortedImagePlaneRight->SetMaterial(0, DistortedDynamicMaterialSelfRight);
    planeActor->UndistortedImagePlaneLeft->SetMaterial(0, UndistortedDynamicMaterialsLeft[ori_img_idx]);
    planeActor->UndistortedImagePlaneRight->SetMaterial(0, UndistortedDynamicMaterialsRight[ori_img_idx]);
    planeActor->DepthImagePlane->SetMaterial(0, DepthDynamicMaterialSelf);
    image_plane = planeActor;
    SetMode(DisplayMode);
    UE_LOG(LogTemp, Warning, TEXT("[ViveSR] DualCameraImageRenderer Initial "));

	SetActorTickEnabled(true);
}
void AViveSR_DualCameraImageRenderer::Release() {
	SetActorTickEnabled(false);
	camera_status = DualCameraStatus::IDLE;
	ViveSR_DualCameraImageCapture::Instance()->Release();
}
// Called when the game starts or when spawned
void AViveSR_DualCameraImageRenderer::BeginPlay()
{
	Super::BeginPlay();
	
	
}

// Called every frame
void AViveSR_DualCameraImageRenderer::Tick(float DeltaTime)
{
    Super::Tick(DeltaTime);
    if (camera_status != DualCameraStatus::WORKING) return;

    if (UpdateDistortedMaterial) {       
        if (!ViveSR_DualCameraImageCapture::Instance()->UpdateDistortedImage())
            return;        
        UpdateSeeThroughImage(DistortedDynamicMaterialsLeft, DistortedDynamicMaterialsRight, need_depurple);
        UpdateDistortedPose();
    }
    if (UpdateUndistortedMaterial) {        
        if (!ViveSR_DualCameraImageCapture::Instance()->UpdateUndistortedImage())
            return;
        UpdateSeeThroughImage(UndistortedDynamicMaterialsLeft, UndistortedDynamicMaterialsRight, need_depurple);
        UpdateUndistortedPose();
    }
    if (UpdateDepthMaterial) {
        UpdateDepthImage();
    }    
}

void AViveSR_DualCameraImageRenderer::UpdateSeeThroughImage(TArray<UMaterialInstanceDynamic*> matListLeft, TArray<UMaterialInstanceDynamic*> matListRight, bool de_purple) {
    int frame_idx, time_idx;
    matListLeft == DistortedDynamicMaterialsLeft ?
        ViveSR_DualCameraImageCapture::Instance()->GetDistortedTexture(ImageLeft, ImageRight, frame_idx, time_idx) :
        ViveSR_DualCameraImageCapture::Instance()->GetUndistortedTexture(ImageLeft, ImageRight, frame_idx, time_idx);
    if (last_texture_update_time == time_idx) return;
    last_texture_update_time = time_idx;
    real_fps = last_texture_update_time * 0.001f;
    if (need_depurple)
        Gamma = ViveSR_DualCameraImageCapture::Instance()->GetGamma();
    for (int i = 0; i < matListLeft.Num(); i++) {
        if (IsValid(matListLeft[i]))
        {
            matListLeft[i]->SetTextureParameterValue(TEXT("Left_Eye"), ImageLeft);
            if (need_depurple) {
                matListLeft[i]->SetScalarParameterValue(TEXT("Gamma"), Gamma);
            }
        }
    }
    for (int i = 0; i < matListRight.Num(); i++) {
        if (IsValid(matListRight[i]))
        {
            matListRight[i]->SetTextureParameterValue(TEXT("Right_Eye"), ImageRight);
            if (need_depurple) {
                matListRight[i]->SetScalarParameterValue(TEXT("Gamma"), Gamma);
            }
        }
    }
}

void AViveSR_DualCameraImageRenderer::UpdateDepthImage() {
    UTexture2D *img;
    int frame_idx, time_idx;
    if (!ViveSR_DualCameraImageCapture::Instance()->UpdateDepthImage()) return;
    ViveSR_DualCameraImageCapture::Instance()->GetDepthTexture(img, frame_idx, time_idx);
    if (last_depth_update_time != time_idx)
    {
        last_depth_update_time = time_idx;
        real_depth_fps = last_depth_update_time * 0.001f;
        for (int i = 0; i < DepthDynamicMaterials.Num(); i++) {
            if (IsValid(DepthDynamicMaterials[i]))
            {
                DepthDynamicMaterials[i]->SetTextureParameterValue(TEXT("Depth"), img);
            }
        }
    }   
}
void AViveSR_DualCameraImageRenderer::UpdateUndistortedPose()
{
	auto pose_position = ViveSR_DualCameraImageCapture::Instance()->GetUndistortedLocalPosition(DualCameraIndex::LEFT);
	auto pose_rotation = ViveSR_DualCameraImageCapture::Instance()->GetUndistortedLocalRotation(DualCameraIndex::LEFT);


	if (IsValid(vr_origin))
	{
		SetActorTransform(vr_origin->GetActorTransform());
		RootComponent->AddLocalTransform(FTransform(pose_rotation, pose_position * 100));
	}
	else
	{
		RootComponent->SetWorldLocationAndRotation(pose_position * 100, pose_rotation);
	}
}

void AViveSR_DualCameraImageRenderer::UpdateDistortedPose()
{
	auto pose_position = ViveSR_DualCameraImageCapture::Instance()->GetDistortedLocalPosition(DualCameraIndex::LEFT);
	auto pose_rotation = ViveSR_DualCameraImageCapture::Instance()->GetDistortedLocalRotation(DualCameraIndex::LEFT);

	FVector local_location(pose_position.X - 0.071, pose_position.Y + 0.0325, pose_position.Z + 0.0026);
	local_location = local_location * 100;

	if (IsValid(vr_origin))
	{
		SetActorTransform(vr_origin->GetActorTransform());
		RootComponent->AddLocalTransform(FTransform(pose_rotation, local_location));
	}
	else
	{
		RootComponent->SetWorldLocationAndRotation(local_location, pose_rotation);
	}
}

UProceduralMeshComponent* AViveSR_DualCameraImageRenderer::GetDistortedImagePlane(DualCameraIndex eye)
{
	if (eye == DualCameraIndex::LEFT)
		return static_cast<AViveSR_DualCameraImagePlane*>(image_plane)->DistortedImagePlaneLeft;
	else
		return static_cast<AViveSR_DualCameraImagePlane*>(image_plane)->DistortedImagePlaneRight;
}
UStaticMeshComponent* AViveSR_DualCameraImageRenderer::GetUndistortedImagePlane(DualCameraIndex eye)
{
	if (eye == DualCameraIndex::LEFT)
		return static_cast<AViveSR_DualCameraImagePlane*>(image_plane)->UndistortedImagePlaneLeft;
	else
		return static_cast<AViveSR_DualCameraImagePlane*>(image_plane)->UndistortedImagePlaneRight;
}

UStaticMeshComponent* AViveSR_DualCameraImageRenderer::GetDepthImagePlane()
{
	return static_cast<AViveSR_DualCameraImagePlane*>(image_plane)->DepthImagePlane;
}
