// Fill out your copyright notice in the Description page of Project Settings.

#include "ViveSR_RigidReconstructionRender.h"
#include "ViveSR_RigidReconstructionData.h"
#include "ViveSR_URigidReconstruction.h"
#include "ViveSR_SceneUnderstanding.h"
#include "ViveSR_RigidReconstructionEnums.h"
#include "ViveSR_Enums.h"
#include "SRWorks.h"
#include "DrawDebugHelpers.h"
#include "Math/UnrealMathUtility.h"
#include <algorithm>

// Sets default values
AViveSR_RigidReconstructionRender::AViveSR_RigidReconstructionRender()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;
	MeshComponent = CreateDefaultSubobject<UProceduralMeshComponent>(TEXT("Mesh"));
	RootComponent = MeshComponent;
	MeshComponent->bUseAsyncCooking = true;
}

// Called when the game starts or when spawned
void AViveSR_RigidReconstructionRender::BeginPlay()
{
	Super::BeginPlay();
	ViveSR_SceneUnderstanding::Instance()->EnableSceneUnderstanding(EnabledSceneUnderstanding);
	if (EnabledSceneUnderstanding) {
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::BED, 10, EnableBed);
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::CEILING, 10, EnableCeiling);
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::CHAIR, 10, EnableChair);
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::FLOOR, 10, EnableFloor);
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::TABLE, 10, EnableTable);
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::WALL, 10, EnableWall);
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::WINDOW, 10, EnableWindow);
		ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::MONITOR, 10, EnableMonitor);
        ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::FURNITURE, 10, EnableFurniture);
        ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::DOOR, 10, EnableDoor);
        ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::PICTURE, 10, EnablePicture);
        ViveSR_SceneUnderstanding::Instance()->SetCustomSceneUnderstandingConfig(ViveSR::RigidReconstruction::SceneUnderstandingObjectType::PERSON, 10, EnablePerson);
	}
    if (DisplayMode == ReconstructionDisplayMode::ADAPTIVE_MESH) {
        GetWorldTimerManager().SetTimer(reconstruction_time_handle, this, &AViveSR_RigidReconstructionRender::Display, 0.2F, true, 0);
        adaptive_range_min = FMath::Pow(2, (int)MinGrizSize + 1);
        adaptive_range_max = FMath::Pow(2, (int)MaxGrizSize + 1);
        UE_LOG(LogTemp, Warning, TEXT("[ViveSR] ADAPTIVE_MAX_GRID %f"), adaptive_range_max);
        UE_LOG(LogTemp, Warning, TEXT("[ViveSR] ADAPTIVE_MIN_GRID %f"), adaptive_range_min);
        if (adaptive_range_min <= adaptive_range_max) {
            int result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterFloat(ViveSR::RigidReconstruction::Param::ADAPTIVE_MAX_GRID, adaptive_range_max* 0.01f);
            UE_LOG(LogTemp, Warning, TEXT("[ViveSR] ADAPTIVE_MAX_GRID %f"), result);
            result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterFloat(ViveSR::RigidReconstruction::Param::ADAPTIVE_MIN_GRID, adaptive_range_min* 0.01f);
            UE_LOG(LogTemp, Warning, TEXT("[ViveSR] ADAPTIVE_MIN_GRID %f"), result);
            result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterFloat(ViveSR::RigidReconstruction::Param::ADAPTIVE_ERROR_THRES, ErrorThres* 0.01f);
            UE_LOG(LogTemp, Warning, TEXT("[ViveSR] ADAPTIVE_ERROR_THRES %f"), result);
        }
    }
    else if(DisplayMode == ReconstructionDisplayMode::FIELD_OF_VIEW) {
        GetWorldTimerManager().SetTimer(reconstruction_time_handle, this, &AViveSR_RigidReconstructionRender::Display, 0.2F, true, 0);
    }
}

void AViveSR_RigidReconstructionRender::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
	Super::EndPlay(EndPlayReason);
	if (ViveSR_URigidReconstruction::Instance()->is_scanning)
		ViveSR_URigidReconstruction::Instance()->StopScanning();
    GetWorldTimerManager().ClearTimer(reconstruction_time_handle);
}

// Called every frame
void AViveSR_RigidReconstructionRender::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
    if (!ViveSR_URigidReconstruction::Instance()->is_scanning) {
        FlushPersistentDebugLines(GetWorld());
    }
    if (EnablePreviewScanModel) {
        MeshComponent->CreateMeshSection_LinearColor(0, vertex_preview, index_preview, normals, UV0, vertex_color_preview, tangents, true);
    }
}

void AViveSR_RigidReconstructionRender::Initial()
{
	ViveSR_URigidReconstruction::Instance()->Initial();
	ViveSR_URigidReconstruction::Instance()->AllocOutputDataMemory();
 
    SetMeshDisplayMode(DisplayMode);
    if (MeshMaterial != nullptr) SetMeshMaterial(MeshMaterial);
       
}
void AViveSR_RigidReconstructionRender::Release() {
	ViveSR_URigidReconstruction::Instance()->Release();
}

bool AViveSR_RigidReconstructionRender::SetMeshDisplayMode(ReconstructionDisplayMode displayMode)
{
	int result = ViveSR::Error::FAILED;
	if (displayMode == ReconstructionDisplayMode::ADAPTIVE_MESH)
	{
		result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterBool(ViveSR::RigidReconstruction::Param::LITE_POINT_CLOUD_MODE, false);
		result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterBool(ViveSR::RigidReconstruction::Param::FULL_POINT_CLOUD_MODE, false);
		result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterBool(ViveSR::RigidReconstruction::Param::LIVE_ADAPTIVE_MODE, true);
	}

    else if (displayMode == ReconstructionDisplayMode::FIELD_OF_VIEW)
    {
        result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterBool(ViveSR::RigidReconstruction::Param::LITE_POINT_CLOUD_MODE, true);
        result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterBool(ViveSR::RigidReconstruction::Param::FULL_POINT_CLOUD_MODE, false);
        result = ViveSR::SRWork::RigidReconstruction::SetReconstructionParameterBool(ViveSR::RigidReconstruction::Param::LIVE_ADAPTIVE_MODE, false);
    }
	return (result == ViveSR::Error::WORK);
}

void AViveSR_RigidReconstructionRender::SetMeshMaterial(UMaterialInterface *material)
{

	MeshComponent->SetMaterial(0, material);
}
bool AViveSR_RigidReconstructionRender::GetisSceneUnderstanding() {
	return EnabledSceneUnderstanding;
}

void AViveSR_RigidReconstructionRender::Display() {
    if (ViveSR_URigidReconstruction::Instance()->is_scanning)
    {
        ViveSR_URigidReconstruction::Instance()->GetRigidReconstructionFrame(processed_frame);
        UE_LOG(LogTemp, Warning, TEXT("[ViveSR] processed_frame %d"), processed_frame);
        UE_LOG(LogTemp, Warning, TEXT("[ViveSR] last_processed_frame %d"), last_processed_frame);
        if (processed_frame != last_processed_frame)
        {
            ViveSR_URigidReconstruction::Instance()->GetRigidReconstructionData(
                processed_frame,
                pose,
                vertex_num,
                vertex_data,
                vertex_in_float,
                index_num,
                index_data);
            UE_LOG(LogTemp, Warning, TEXT("[ViveSR] vertex_num %d"), vertex_num);
            UE_LOG(LogTemp, Warning, TEXT("[ViveSR] index_num %d"), index_num);
            if (vertex_num > 0)
            {

                valid_vertex_data.Init(FVector::ZeroVector, vertex_num);
                point_cloud_color.Init(FColor::White, vertex_num);
                TArray<FLinearColor> vertex_color;
                FLinearColor linear_color;
                color_set = new uint8[4];
                for (int i = 0; i < vertex_num; i++)
                {
                    float x = vertex_data[i * 4 + 2];
                    float y = vertex_data[i * 4 + 0];
                    float z = vertex_data[i * 4 + 1];
                    valid_vertex_data[i].Set(x * 100, y * 100, z * 100);
                    FColor color;
                    memcpy(color_set, (vertex_data.GetData() + i * 4 + 3), 4);
                    color.R = color_set[0];
                    color.G = color_set[1];
                    color.B = color_set[2];
                    color.A = color_set[3];
                    if (DisplayMode == ReconstructionDisplayMode::FIELD_OF_VIEW) {
                        point_cloud_color[i] = color;
                    }
                    if (EnabledSceneUnderstanding) {
                        linear_color = FLinearColor(color);
                    }
                    else {
                        linear_color = FLinearColor(0, 0.2357, 0.943, 0);
                    }
                    vertex_color.Add(linear_color);
                }
                delete[] color_set;
                color_set = nullptr;
                if (DisplayMode == ReconstructionDisplayMode::ADAPTIVE_MESH) {
                    TArray<int> valid_index_data;
                    valid_index_data.Init(0, index_num);
                    for (int i = 0; i < index_num; i += 3)
                    {
                        valid_index_data[i + 0] = index_data[i + 0];
                        valid_index_data[i + 1] = index_data[i + 2];
                        valid_index_data[i + 2] = index_data[i + 1];
                    }
                    MeshComponent->CreateMeshSection_LinearColor(0, valid_vertex_data, valid_index_data, normals, UV0, vertex_color, tangents, true);
                    index_preview.Init(0, index_num);
                    std::copy(valid_index_data.GetData(), valid_index_data.GetData() + valid_index_data.Num(), index_preview.GetData());
                }
                vertex_preview.Init(FVector::ZeroVector, vertex_num);
                std::copy(valid_vertex_data.GetData(), valid_vertex_data.GetData() + valid_vertex_data.Num(), vertex_preview.GetData());
                vertex_color_preview.Init(FLinearColor::Blue, vertex_num);
                std::copy(vertex_color.GetData(), vertex_color.GetData() + vertex_color.Num(), vertex_color_preview.GetData());
            }
            last_processed_frame = processed_frame;
        }
        if (DisplayMode == ReconstructionDisplayMode::FIELD_OF_VIEW) {
            for (int k = 0; k < valid_vertex_data.Num(); k++) {
                DrawDebugPoint(GetWorld(), valid_vertex_data[k], 22, point_cloud_color[k], false, 0.2F, 0);
            }
        }
    }
    if (ViveSR_URigidReconstruction::Instance()->is_exporting) {
        ViveSR_URigidReconstruction::Instance()->UpdateExportProgress();
    }
    if (ViveSR_SceneUnderstanding::Instance()->is_exporting) {
        ViveSR_SceneUnderstanding::Instance()->UpdateSceneUnderstandingProgress();
    }
}